﻿using System;
using System.Windows.Forms;
using ELTE.BoardGame.Model;
using ELTE.BoardGame.Persistence;
using ELTE.BoardGame.View;

namespace ELTE.BoardGame.Controller
{
    public class BoardGameController
    {
        #region Private fields

        private BoardGameModel model; // játékmodell
        private BoardGameView view; // (fő) nézet
        private SaveFileDialog saveFileDialog; // dialógusok
        private OpenFileDialog openFileDialog;

        #endregion

        #region Constructors

        /// <summary>
        /// Táblajáték prezentáció példányosítása.
        /// </summary>
        /// <param name="model">A játékmodell.</param>
        public BoardGameController(BoardGameModel model)
        {
            if (model == null)
                throw new ArgumentNullException(nameof(model), "Nincs megadva játékmodell.");

            this.model = model;

            this.model.FieldChanged += new EventHandler<FieldChangedEventArgs>(Model_FieldChanged);
            this.model.GameOver += new EventHandler(Model_GameOver);
            this.model.GameWon += new EventHandler<GameWonEventArgs>(Model_GameWon);

            this.view = new DrawingForm();
            this.view.KeyPreview = true;
            this.view.Shown += View_Shown;
            this.view.KeyDown += View_KeyDown;
            this.view.Resize += View_Resize;
            this.view.BoardClicked += View_BoardClicked;
        }

        #endregion

        #region Public properties

        /// <summary>
        /// Nézet lekérdezése.
        /// </summary>
        public BoardGameView View { get { return this.view; } }

        #endregion

        #region Model event handlers

        /// <summary>
        /// Játék megnyerésének eseménykezelése.
        /// </summary>
        private void Model_GameWon(object sender, GameWonEventArgs e)
        {
            switch (e.Player)
            {
                case PlayerData.PlayerO:
                    MessageBox.Show("A kör játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    break;
                case PlayerData.PlayerX:
                    MessageBox.Show("A kereszt játékos győzött!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    break;
            }
            this.model.NewGame();
            this.view.UpdateViewState(this.GetSessionState());
        }

        /// <summary>
        /// Játék végének eseménykezelése.
        /// </summary>
        private void Model_GameOver(object sender, EventArgs e)
        {
            MessageBox.Show("Döntetlen játék!", "Játék vége!", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            this.model.NewGame();
            this.view.UpdateViewState(this.GetSessionState());
        }

        /// <summary>
        /// Modell mezőváltozásának eseménykezelése.
        /// </summary>
        private void Model_FieldChanged(object sender, FieldChangedEventArgs e)
        {
            this.view.UpdateViewState(this.GetSessionState());
        }

        #endregion

        #region View event handlers

        /// <summary>
        /// Nézet megjelenítésének eseménykezelője.
        /// </summary>
        private void View_Shown(object sender, EventArgs e)
        {
            this.view.UpdateViewState(this.GetSessionState());
        }

        /// <summary>
        /// Táblabeli kattintás eseménykezelője.
        /// </summary>
        private void View_BoardClicked(object sender, BoardEventArgs e)
        {
            Int32 x = e.Column * this.model.BoardWidth / this.view.BoardWidth;
            Int32 y = e.Row * this.model.BoardHeight / this.view.BoardHeight;

            this.model.StepGame(x, y);
        }

        /// <summary>
        /// Billentyűzet lenyomásának eseménykezelője.
        /// </summary>
        private void View_KeyDown(object sender, KeyEventArgs e)
        {
            if (!e.Control) // ha nincs lenyomva a Control, nem csinálunk semmit
                return;

            switch (e.KeyCode)
            {
                case Keys.N: // új játék indítása (Ctrl+N)
                    this.model.NewGame();
                    break;
                case Keys.L: // betöltés (Ctrl+L)
                    this.LoadGame();
                    break;
                case Keys.S: // mentés (Ctrl+S)
                    this.SaveGame();
                    break;
            }

            this.view.UpdateViewState(this.GetSessionState());
        }

        /// <summary>
        /// Nézet átméretezésének eseménykezelője.
        /// </summary>
        private void View_Resize(object sender, EventArgs e)
        {
            this.view.UpdateViewState(this.GetSessionState());
        }

        #endregion

        #region Private methods

        /// <summary>
        /// Játék betöltése.
        /// </summary>
        private void LoadGame()
        {
            if (this.openFileDialog == null)
            {
                this.openFileDialog = new OpenFileDialog();
                this.openFileDialog.Filter = "Játékmentések|*.sav";
                this.openFileDialog.Title = "Tic-Tac-Toe - Játék betöltése";
            }

            if (this.openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.model.LoadGame(this.openFileDialog.FileName);
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a betöltés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        
        /// <summary>
        /// Játék mentése.
        /// </summary>
        private void SaveGame()
        {
            if (this.saveFileDialog == null)
            {
                this.saveFileDialog = new SaveFileDialog();
                this.saveFileDialog.Filter = "Játékmentések|*.sav";
                this.saveFileDialog.Title = "Tic-Tac-Toe - Játék mentése";
            }

            if (this.saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    this.model.SaveGame(this.saveFileDialog.FileName);
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a mentés során.", "Tic-Tac-Toe", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        
        /// <summary>
        /// Munkafolyamat állapot lekérdezése.
        /// </summary>
        private PlayerData[,] GetSessionState()
        {
            PlayerData[,] data = new PlayerData[this.model.BoardHeight, this.model.BoardWidth];

            for (Int32 i = 0; i < this.model.BoardWidth; i++)
                for (Int32 j = 0; j < this.model.BoardHeight; j++)
                {
                    data[i, j] = this.model[i, j];
                }

            return data;
        }

        #endregion
    }
}
